﻿-- Author      : RisM
-- Create Date : 6/28/2009 4:03:23 PM

local SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L			= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)
local HELPFILE	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell_HELPFILE",			false)

SpeakinSpell:PrintLoading("slashcommands.lua")

-------------------------------------------------------------------------------
-- SLASH COMMAND HANDLERS
-------------------------------------------------------------------------------


function SpeakinSpell:RegisterChatCommands()
	-- add the localized slash commands (support both)
	for _,command in pairs(SpeakinSpell.SlashCommands) do
		--local lkey = "/"..command -- no, but this is how it is for the keys to the next parameter in the input
		local lkey = command -- for this command prefix the Locale table specifies keys matching values in English
		local lcommand = L[lkey] -- returns lkey if not defined in Locale file
		if lcommand ~= lkey then
			tinsert(SpeakinSpell.SlashCommands, lcommand)
		end
	end
	
	-- register all chat commands
	for k,v in pairs(SpeakinSpell.SlashCommands) do
		self:RegisterChatCommand(v,"OnSlashCommand")
	end
end


-- return true if this is a "/ss ad /w playername"
-- return false otherwise
function SpeakinSpell:ProcessAdWhisper(input)
	-- check for /ss ad /w playername
	local whisperad = self:StartsWith(input, "ad /w")
	local len = string.len("ad /w")
	if not whisperad then -- Localized check
		whisperad = self:StartsWith(input, L["/ad /w"])
		len = string.len(L["/ad /w"])
	end
	if whisperad then
		local target = string.sub(input, len+2, -1) --from len to the end of the string
		self:Advertise("WHISPER", target)
		return true
	end
	
	-- this was not "/ss ad /w playername"
	return false
end


function SpeakinSpell:ProcessUserMacro(input)
	local macro = self:StartsWith(input, "macro")
	if not macro then -- Localized check
		macro = self:StartsWith(input, L["/macro"])
	end
	
	-- if it's still not a macro, then we're done
	if not macro then
		return false
	end
	
	self:DebugMsg(funcname, "User Macro input:"..tostring(input))
	
	local DetectedEventStub = {
		-- event descriptors
		name = input, -- use the entire lowercase input = "macro something"
		type = "MACRO",
		-- event-specific data for substitutions
		caster = UnitName("player"),
	}
	
	-- NOTE: do not inherit parent event data here
	--		that would corrupt the event key,
	--		causing us to load the wrong speech list and options
	--		and leading to infinite recursion
	--		see SpeakinSpell:InheritParentEvent()
	
	-- signal the event
	self:OnSpeechEvent( DetectedEventStub )
	return true
end


function SpeakinSpell:GetETEForSSMacro(input)
	local funcname = "GetETEForSSMacro"
	
	local macro = input
	for _,slash in pairs(SpeakinSpell.SlashCommands) do
		if self:StartsWith(macro, slash) then
			macro = string.sub(macro, string.len(slash)+2)
			break
		end
	end
	while self:StartsWith(macro, " ") do
		macro = string.sub(macro, 2)
	end
	if not ( self:StartsWith(macro, "macro") or self:StartsWith(macro, L["/macro"]) ) then
		-- not a recursive macro
		--self:DebugMsg(funcname, string.format([["%s" doesn't start with "macro"]], macro) )
		return nil
	end
	
	local key = self:Keyify( "MACRO"..tostring(macro) )
	--self:DebugMsg(funcname, "key:"..key)
	return self:GetActiveEventTable()[key]
end


function SpeakinSpell:ProcessRecursiveUserMacro(input, ParentDetectedEvent)
	local funcname = "ProcessRecursiveUserMacro"

	local ete = self:GetETEForSSMacro(input)
	if not ete then
		-- this is not an error, because we call this function as a test
		-- but it does mean we have nothing to do here
		--self:ErrorMsg(funcname, "no settings defined for "..tostring(input))
		return false
	end
		
	if self.RuntimeData.RecursiveCall == input then
		self:ErrorMsg(funcname, "stopped infinite recursive call for "..tostring(input))
		return true --consider this processed and stop
	end
	
	self.RuntimeData.RecursiveCall = input
	self:DebugMsg(funcname, "Recursive User Macro input: "..tostring(self.RuntimeData.RecursiveCall))
	
	local DetectedEventStub = {
		-- event descriptors
		name = ete.DetectedEvent.name, -- use the entire lowercase input = "macro something"
		type = "MACRO",
		-- event-specific data for substitutions
		target = ParentDetectedEvent.target,
		caster = ParentDetectedEvent.caster,
	}
	
	-- signal the event
	self:OnRecursiveEventDetected( input, DetectedEventStub, ParentDetectedEvent )
	
	-- clear the infinite recursive call blocker
	self.RuntimeData.RecursiveCall = nil
	
	return true
end


function SpeakinSpell:OnSlashCommand(input)
	local funcname = "OnSlashCommand"

	input = string.lower(input)
	self:DebugMsg(funcname,"OnSlashCommand:"..tostring(input))
	
	-- if nothing is specified, then choose a default command
	if input == nil or input == "" then
		input = "toggle"
	else
		-- check for commands that require special parsing
		
		-- check for /ss ad /w playername
		if self:ProcessAdWhisper(input) then
			return
		end
		
		-- check for /ss macro something
		if self:ProcessUserMacro(input) then
			return
		end
		
	end
	
	-- table of supporting slash commands
	-- NOTE: support both English and current locale
	--	non-English game clients can typically use both translated and untranslated slash commands
	--	for built-in functions like /bg chat, etc, so it makes sense here as well
	local SlashCommandTable = {
		["toggle"]		= function() self:ShowOptions_Toggle() end,
		["options"]		= function() self:ShowOptions() end,
		["create"]		= function() self:ShowCreateNew() end,
		["messages"]	= function() self:ShowMessageOptions() end,
		["random"]		= function() self:ShowRandomSubsOptions() end,
		["import"]		= function() self:ShowImport() end,
		["colors"]		= function() self:ShowColorsGUI() end,
		["network"]		= function() self:ShowNetworkOptions() end,
		["help"]		= function() self:ShowHelp() end,
		["reset"]		= function() self:ResetToDefaults(nil) end,
		["advertise"]	= function() self:Advertise("") end,
		["ad /s"]		= function() self:Advertise("SAY") end,
		["ad /p"]		= function() self:Advertise("PARTY") end,
		["ad /ra"]		= function() self:Advertise("RAID") end,
		["testallsubs"] = function() self:TestAllSubs() end,
		["memory"]		= function() self:Print( self:GetAddonMemoryUsedString() ) end,
		["sync"]		= function() self:Network_AutoSync() end,
		["sync version"]= function() self:Network_SendVersionRequest() end,
		["sync new"]	= function() self:Network_SendNewEvents(nil, true, nil) end,
		["sync speeches"] = function() 
			self:Network_SendEventTable(nil) 
			self:Network_BroadcastCommand("ET?",nil)
		end,
	}
	
	-- add localized commands
	for command,Func in pairs(SlashCommandTable) do
		local lcommandkey = "/"..tostring(command)
		local lcommand = L[lcommandkey] --always returns something, if even just lcommandkey
		if lcommand ~= lcommandkey then
			SlashCommandTable[ L["/"..tostring(command)] ] = Func
		end
	end
	
	-- what did the user enter?
	Func = SlashCommandTable[input]
	
	-- process the command
	if Func then
		Func(input)
	else -- in all other cases, show help
		-- go straight to the slash commands page
		-- NOTE: A Blizzard Bug won't open a tier 3 page unless the tree has already been expanded to show tier 2
		--		so we'll try to show the parent frame first
		-- NOTE: I tried to work aroubd it by first opening the root page, but it stayed there
		--		I guess there's a timing issue with the windows messages (?) that makes it stick on the first one
		--		or maybe you're not allowed and it stops executing when you call this function (?)
		self:ShowHelp() -- whatever, it didn't work, see notes above
		-- the slash commands page is now tier 2 to avoid this issue and other general bugginess with tier 3 pages
		--InterfaceOptionsFrame_OpenToCategory(HELPFILE.SLASH_COMMANDS)
		self:HelpPages_SelectTopic( HELPFILE.SLASH_COMMANDS )
	end
end


